#include "EventManager.h"
#include <algorithm>
#include <iostream>
#include "property/MyPropertySystem.h"

EventManager::EventManager()
{
}

bool EventManager::AddEvent(EventId eventId, EventFunc eventFunc)
{
    return AddEvent(eventId, eventFunc, eventId);
}

bool EventManager::AddEvent(EventId eventId, EventFunc eventFunc, EntryName entryName)
{
    EventHandler handler = { eventId, eventFunc, entryName };

    auto iter = m_eventMap.find(eventId);
    if (iter != m_eventMap.end())
    {
        EventList& list = iter->second;

        auto it = std::find_if(list.begin(), list.end(), [&entryName](EventHandler const& item) -> bool {
            return item.entryName.compare(entryName) == 0;
        });

        if (it != list.end())
        {
            return false;
        }
        list.push_back(handler);
        m_entryMap[entryName] = eventId;
        return true;
    }

    EventList list;
    list.push_back(handler);
    m_eventMap[eventId] = list;
    m_entryMap[entryName] = eventId;
    return true;
}

bool EventManager::RemoveEvent(EntryName entryName)
{
    auto idIter = m_entryMap.find(entryName);
    if (idIter == m_entryMap.end())
        return false;

    auto iter = m_eventMap.find(idIter->second);
    if (iter != m_eventMap.end())
    {
        EventList& list = iter->second;
        auto it = std::find_if(list.begin(), list.end(), [&entryName](EventHandler const& item) -> bool {
            return item.entryName.compare(entryName) == 0;
        });

        if (it != list.end())
        {
            list.erase(it);
            return true;
        }
    }

    return false;
}

void EventManager::Update(EventId eventId, const MyPropertySystem& param)
{
    auto iter = m_eventMap.find(eventId);
    if (iter != m_eventMap.end())
    {
        EventList& list = iter->second;

        for (const EventHandler& item : list)
        {
            item.func(param);
        }
    }
}