//MyPropertySystem.cpp
#include <string>
#include <map>
#include <list>
#include <vector>
#include <functional>
#include "MyPropertySystem.h"

std::string  ToString(IProperty* property)
{
    if (property == nullptr)
        return "";

    std::function<std::string(void*)> fn[ept_string+1];
    fn[ept_bool] = [](void* value)
    {
        bool* v = static_cast<bool*>(value);
        return std::to_string(*v);
    };
    fn[ept_int] = [](void* value)
    {
        int* v = static_cast<int*>(value);
        return std::to_string(*v);
    };
    fn[ept_float] = [](void* value)
    {
        float* v = static_cast<float*>(value);
        return std::to_string(*v);
    };
    fn[ept_string] = [](void* value)
    {
        std::string* v = static_cast<std::string*>(value);
        return *v;
    };

    int type = property->GetType();
    if (type > (int)ept_string)
        return "";

    return fn[type](property->GetData());
}

MyPropertySystem::MyPropertySystem()
{
}

MyPropertySystem::MyPropertySystem(const MyPropertySystem& propertySystem)
{
    propertySystem.CopyToPropertyList(&m_properties);
}

template <typename T>
void MakeProperty(PropertyList* newPropertyList, const IProperty* prop, T zero)
{
    MyProperty<T>* newProp = new MyProperty<T>(prop->GetName());
    const MyProperty<T>* oldProp = dynamic_cast<const MyProperty<T>*>(prop);
    if (oldProp)
        *newProp = *oldProp;
    else
        *newProp = zero;
    newPropertyList->push_back(newProp);
}

void MyPropertySystem::CopyToPropertyList(PropertyList* propertyList) const
{
    const PropertyList& list = m_properties;
    PropertyList::const_iterator it = list.begin();

    std::function<void(const IProperty*)> fn[ept_string+1];
    fn[ept_bool] = [propertyList](const IProperty* prop)  
    { 
        MakeProperty<bool>(propertyList, prop, false);
    };
    fn[ept_int] = [propertyList](const IProperty* prop)
    {
        MakeProperty<int>(propertyList, prop, 0);
    };
    fn[ept_float] = [propertyList](const IProperty* prop)
    {
        MakeProperty<float>(propertyList, prop, 0.0f);
    };
    fn[ept_string] = [propertyList](const IProperty* prop)
    {
        MakeProperty<std::string>(propertyList, prop, "");
    };

    while (it != m_properties.end())
    {
        const IProperty* property = (*it);

        int type = property->GetType();
        if (type > (int)ept_string)
            continue;
        fn[type](property);
        ++it;
    }
}


MyPropertySystem::~MyPropertySystem()
{
    {
        PropertyList::iterator it = m_properties.begin();
        PropertyList::iterator end = m_properties.end();
        for( ; it != end; ++it )
        {
            IProperty* prop = (*it);
            delete prop;
        }
        m_properties.clear();
    }
}
//-------------------------------------------------------------------------------------------------------
IProperty* MyPropertySystem::GetProperty( const std::string& name )
{
    PropertyList::iterator it = m_properties.begin();
    while( it != m_properties.end() )
    {
        if( name.compare((*it)->GetName()) == 0 )
        {
            IProperty* prop = (IProperty*)(*it);
            return prop;
        }
        ++it;
    }
    return nullptr;
}